package de.fhdw.gaming.ipspiel23.tictactoe.core.domain.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.Optional;

import de.fhdw.gaming.core.domain.Move;
import de.fhdw.gaming.ipspiel23.gst.domain.IKopplung;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeField;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToePlayer;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.evaluation.IEvaluationStrategy;
import de.fhdw.gaming.ipspiel23.tictactoe.core.evaluation.MixedEvaluationStrategy;
import de.fhdw.gaming.ipspiel23.tictactoe.core.moves.impl.TicTacToeDefaultMoveFactory;

/**
 * Implementation of the {@link IKopplung} interface for the Tic-Tac-Toe game.
 * Provides methods for obtaining possible moves, evaluating the game state,
 * and getting the current player.
 * 
 * @author borkowitz
 */
public class TicTacToeKopplungImpl  implements IKopplung<TicTacToePlayer, TicTacToeState> {

    /**
     * The evaluation strategy used to evaluate the Tic-Tac-Toe game state.
     */
    private final IEvaluationStrategy evaluationStrategy;

    /**
     * Constructs a new TicTacToeKopplungImpl object with a mixed evaluation strategy.
     */
    public TicTacToeKopplungImpl() {
        this.evaluationStrategy = new MixedEvaluationStrategy();
    }
    
    /**
     * Retrieves a collection of possible moves for the given Tic-Tac-Toe game state.
     * 
     * @param state The current Tic-Tac-Toe game state.
     * @return An optional containing the collection of possible moves.
     */
    @Override
    public Optional<Collection<Move<TicTacToePlayer, TicTacToeState>>> getPossibleMoves(
            final TicTacToeState state) {
        
        final List<TicTacToeField> fields = new ArrayList<>(
                state.getBoard().getFieldsBeing(TicTacToeFieldState.EMPTY).values());

        if (fields.isEmpty()) {
            return Optional.of(new ArrayList<Move<TicTacToePlayer, TicTacToeState>>());
        }

        final Collection<Move<TicTacToePlayer, TicTacToeState>> possibleMoves = new ArrayList<>();
        final TicTacToeDefaultMoveFactory factory = new TicTacToeDefaultMoveFactory();

        for (final TicTacToeField ticTacToeField : fields) {
            possibleMoves.add(factory.createPlaceMarkMove(state.getCurrentPlayer().isUsingCrosses(),
                    ticTacToeField.getPosition()));

        }

        return Optional.of(possibleMoves);
    }

    /**
     * Evaluates the given Tic-Tac-Toe game state using the configured evaluation strategy.
     * 
     * @param state The current Tic-Tac-Toe game state.
     * @return An optional containing the evaluation result.
     */
    @Override
    public Optional<Integer> evalState(final TicTacToeState state) {
        return this.evaluationStrategy.evaluateState(state);
    }

    /**
     * Retrieves the current player in the given Tic-Tac-Toe game state.
     * 
     * @param state The current Tic-Tac-Toe game state.
     * @return An optional containing the
     */
    @Override
    public Optional<TicTacToePlayer> getCurrentPlayer(final TicTacToeState state) {
        return Optional.of(state.getCurrentPlayer());
    }

    @Override
    public Optional<Boolean> getIsGameOver(final TicTacToeState state) {
        return Optional.of(state.computeNextPlayers().isEmpty());
    }
}
