package de.fhdw.gaming.ipspiel23.tictactoe.core.evaluation;

import java.util.Optional;

import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeState;

/**
 * A mixed evaluation strategy for evaluating the state of a Tic-Tac-Toe game.
 * This strategy combines the evaluations from two different 
 * evaluation strategies: WinEvaluationStrategy and SimpleEvaluationStrategy.
 * The evaluation result is the sum of the evaluations from both strategies.
 * 
 * The MixedEvaluationStrategy is initialized with instances of WinEvaluationStrategy and SimpleEvaluationStrategy.
 * 
 * @see IEvaluationStrategy
 * @see WinEvaluationStrategy
 * @see SimpleEvaluationStrategy
 * @see TicTacToeState
 * @see java.util.Optional
 *
 */
public class MixedEvaluationStrategy implements IEvaluationStrategy {

    /**
     * The evaluation strategy for win states.
     */
     private final IEvaluationStrategy winStrategy;
     
     /**
      * The evaluation strategy for non-win states.
      */
     private final IEvaluationStrategy simpleStrategy;
     
     /**
      * Constructs a new MixedEvaluationStrategy object with instances 
      * of WinEvaluationStrategy and SimpleEvaluationStrategy.
      */
     public MixedEvaluationStrategy() {
          this.winStrategy = new WinEvaluationStrategy();
          this.simpleStrategy = new SimpleEvaluationStrategy();
     }
     
     /**
      * Evaluates the given Tic-Tac-Toe game state using the mixed evaluation strategy.
      * The evaluation result is the sum of the evaluations from the win strategy and the simple strategy.
      * 
      * @param state The Tic-Tac-Toe game state to be evaluated.
      * @return An optional containing the evaluation result as an integer value.
      */
     @Override
     public Optional<Integer> evaluateState(final TicTacToeState state) {

          try {

               final int winEval = this.winStrategy.evaluateState(state).get();
               final int simpleEval = this.simpleStrategy.evaluateState(state).get();
               final int totalEval = winEval + simpleEval;
               
               return Optional.of(totalEval);

          } catch (Exception e) {
               return Optional.empty();
          }
     }
}
