package de.fhdw.gaming.ipspiel23.tictactoe.core.evaluation;

import java.util.Optional;

import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeField;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeFieldState;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToePosition;
import de.fhdw.gaming.ipspiel23.tictactoe.core.domain.TicTacToeState;

/**
 * A simple evaluation strategy for evaluating the state of a Tic-Tac-Toe game.
 * This strategy assigns scores to different game states based on the arrangement of marks on the board.
 * 
 * The evaluation score is determined by counting the number of marks in each row, column, and diagonal.
 * A higher score indicates a more favorable state for the current player.
 * 
 * The evaluation scores are predefined and can be adjusted by modifying the scoreLine() method.
 * 
 * @see IEvaluationStrategy
 * @see TicTacToeState
 *
 */
public class SimpleEvaluationStrategy implements IEvaluationStrategy {

    /**
     * Evaluates the given Tic-Tac-Toe game state using the simple evaluation strategy.
     * The evaluation score is determined by counting the number of marks in each 
     * row, column, and diagonal,
     * and assigning predefined scores based on the counts.
     * Too complex, but not easily simplifyable. Warning suppressed.
     * 
     * @param state The Tic-Tac-Toe game state to be evaluated.
     * @return An optional containing the evaluation score as an integer value.
     */
    @Override
    @SuppressWarnings({ "checkstyle:CyclomaticComplexity", "PMD.CyclomaticComplexity",
        "PMD.CognitiveComplexity" })
    public Optional<Integer> evaluateState(final TicTacToeState state) {
          int rowSum;
          int colSum;
          int diagSum1;
          int diagSum2;
          int score = 0;

          final int boardSize = state.getBoard().getSize();
          for (int i = 0; i < boardSize; i++) {
               rowSum = 0;
               colSum = 0;
               for (int j = 0; j < boardSize; j++) {
                    final TicTacToeField rField = state.getBoard().getFieldAt(TicTacToePosition.of(i, j));
                    final TicTacToeField cField = state.getBoard().getFieldAt(TicTacToePosition.of(j, i));

                    if (rField.getState().equals(TicTacToeFieldState.CROSS)) {
                         rowSum += 1;
                    } else if (rField.getState().equals(TicTacToeFieldState.NOUGHT)) {
                         rowSum -= 1;
                    }

                    if (cField.getState().equals(TicTacToeFieldState.CROSS)) {
                         colSum += 1;
                    } else if (cField.getState().equals(TicTacToeFieldState.NOUGHT)) {
                         colSum -= 1;
                    }
                    score += scoreLine(rowSum, boardSize);
                    score += scoreLine(colSum, boardSize);

               }

          }

          diagSum1 = 0;
          diagSum2 = 0;

          for (int i = 0; i < boardSize; i++) {

               final TicTacToeField dAscField = state.getBoard().getFieldAt(TicTacToePosition.of(i, i));
               final TicTacToeField dDescField = state.getBoard().
                         getFieldAt(TicTacToePosition.of(i, boardSize - i - 1));

               if (dAscField.getState().equals(TicTacToeFieldState.CROSS)) {
                    diagSum1 += 1;
               } else if (dAscField.getState().equals(TicTacToeFieldState.NOUGHT)) {
                    diagSum1 -= 1;
               }
               if (dDescField.getState().equals(TicTacToeFieldState.CROSS)) {
                    diagSum2 += 1;
               } else if (dDescField.getState().equals(TicTacToeFieldState.NOUGHT)) {
                    diagSum2 -= 1;
               }

          }
          score += scoreLine(diagSum1, boardSize);
          score += scoreLine(diagSum2, boardSize);

               return Optional.of(score);
          
     }

    /**
     * Determines the score for a line based on the number of marks and the board size.
     * The predefined scores are as follows:
     * - If the line has all marks of the current player: 100 (winning state)
     * - If the line has all marks of the opponent player: -100 (losing state)
     * - If the line has one mark less than the board size 
     * and all other marks are of the current player: 10 (advantage state)
     * - If the line has one mark less than the board size 
     * and all other marks are of the opponent player: -10 (disadvantage state)
     * - Otherwise: 0 (neutral state)
     * 
     * @param lineSum The sum of marks in the line.
     * @param boardSize The size of the game board.
     * @return The score for the line.
     */
     private int scoreLine(final int lineSum, final int boardSize) {
          if (lineSum == boardSize) {
               return 100;
          }
          if (lineSum == -boardSize) {
               return -100;
          }
          if (lineSum == boardSize - 1) {
               return 10;
          }
          if (lineSum == -boardSize + 1) {
               return -10;
          }
          return 0;
     }
}
